import Flutter
import UIKit

import Dvotemobile

public class SwiftDvotePlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "dvote", binaryMessenger: registrar.messenger())
    let instance = SwiftDvotePlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    var error: NSError?

    switch call.method {
      case "generateMnemonic": 
        if let arg = call.arguments as? Int {
          let mnemonic = Dvotemobile.DvotemobileGenerateMnemonic(arg, &error)
          if error != nil {
            result("Error: Unable to generate a mnemonic")
          }
          result(mnemonic)
        }
        else {
          result("Error: size should be an integer")
        }
        break
        
      case "mnemonicToPrivateKey": 
        let args = call.arguments as? [String]
        let mnemonic = args?[0], hdPath = args?[1]
        if mnemonic != nil && hdPath != nil {
          let privKey = Dvotemobile.DvotemobileMnemonicToPrivateKey(mnemonic, hdPath, &error)
          if error != nil {
            result("Error: Unable to generate a mnemonic")
          }
          result(privKey)
        }
        else {
          result("Error: the argument must be a string array with 2 elements")
        }
        break
        
      case "mnemonicToPublicKey":
        let args = call.arguments as? [String]
        let mnemonic = args?[0], hdPath = args?[1]
        if mnemonic != nil && hdPath != nil {
          let pubKey = Dvotemobile.DvotemobileMnemonicToPublicKey(mnemonic, hdPath, &error)
          if error != nil {
            result("Error: Unable to generate a mnemonic")
          }
          result(pubKey)
        }
        else {
          result("Error: the argument must be a string array with 2 elements")
        }
        break
        
      case "mnemonicToAddress":
        let args = call.arguments as? [String]
        let mnemonic = args?[0], hdPath = args?[1]
        if mnemonic != nil && hdPath != nil {
          let address = Dvotemobile.DvotemobileMnemonicToAddress(mnemonic, hdPath, &error)
          if error != nil {
            result("Error: Unable to generate a mnemonic")
          }
          result(address)
        }
        else {
          result("Error: the argument must be a string array with 2 elements")
        }
        break
        
      case "getPlatformVersion":
        result("iOS " + UIDevice.current.systemVersion)
        break
      default:
        result("404")
    }
  }
}
