import 'dart:async';

import 'package:flutter/services.dart';

final defaultHdPath = "m/44'/60'/0'/0/0";

class Dvote {
  static const MethodChannel _channel = const MethodChannel('dvote');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  /// Generate a random private key and encode it with a human readable BIP39 mnemonic.
  /// Entropy is the bit count (128, ..., 256)
  static Future<String> generateMnemonic(int size) async {
    final String mnemonic =
        await _channel.invokeMethod<String>('generateMnemonic', size);
    return mnemonic;
  }

  /// Return the private key that corresponds to the given mnemonic and (optional) HD derivation path
  static Future<String> mnemonicToPrivateKey(
      String mnemonic, String hdPath) async {
    if (hdPath == null || hdPath == "") hdPath = defaultHdPath;
    final String privKey = await _channel
        .invokeMethod<String>('mnemonicToPrivateKey', [mnemonic, hdPath]);
    return privKey;
  }

  /// Return the public key that corresponds to the given mnemonic and (optional) HD derivation path
  static Future<String> mnemonicToPublicKey(
      String mnemonic, String hdPath) async {
    if (hdPath == null || hdPath == "") hdPath = defaultHdPath;
    final String pubKey = await _channel
        .invokeMethod<String>('mnemonicToPublicKey', [mnemonic, hdPath]);
    return pubKey;
  }

  /// Return the Ethereum address that corresponds to the given mnemonic and (optional) HD derivation path
  static Future<String> mnemonicToAddress(
      String mnemonic, String hdPath) async {
    if (hdPath == null || hdPath == "") hdPath = defaultHdPath;
    final String addr = await _channel
        .invokeMethod<String>('mnemonicToAddress', [mnemonic, hdPath]);
    return addr;
  }
}
